/*! \file globals.h
    \brief Enthlt die Definition der globalen Variablen
 */
#ifndef __GLOBALS_H
#define __GLOBALS_H

/*! \var double *beta
    \brief OUTPUT - Parameter der Funktion \ref mrqmin

    inhomogener Vektor des linearen Systems 
*/
double *beta;

/*! \var double *a;
    \brief INPUT bzw. OUTPUT - Parameter der Funktion \ref mrqmin

    INPUT: Feld der Modellparameter (guessed values)
    OUTPUT: Feld der optimierten Fit-Parameter
*/
double *a;
/*! \var double *aalt
    \brief Wird dazu verwendet einen Genauigkeitsvergleich der Fit-Parameter whrend der Iteration durchzufhren.
 */
double *aalt;
/*! \var double **covar
    \brief OUTPUT-Parameter der Funktion \ref mrqmin
    
    Kovarianz-Matrix
*/
double **covar; 
/*! \var double **alpha
    \brief OUTPUT-Parameter der Funktion \ref mrqmin

    Koeffizientenmatrix des linearen Systems
*/
double **alpha;
/*! \var double chisq
    \brief OUTPUT - Parameter der Funktion \ref mrqmin

    gewichtete Fehlerquadratsumme
*/
double chisq;
/*! \var double ochisq
    \brief OUTPUT - Parameter der Funktion \ref mrqmin

    gewichtete Fehlerquadratsumme des vorherigen Iterationsschrittes.
 */
double ochisq;
/*! \var double *chisqverlauf
    \brief Feld, in dem whrend der Iteration der Verlauf der gewichteten Fehlerquadratsumme gespeichert wird.
 */
double *chisqverlauf;
/*! \var double gen
    \brief Die Genauigkeit bis zu der die Auswertung erfolgen soll.
*/
double gen;
/*! \var double var
    \brief Die Varianz welche aus der Kovarianzmatrix berechnet wird.
 */
double var;
/*! \var double sdvar
    \brief Die Standardabweichung der Varianz
 */
double sdvar;
/*! \var double delta
    \brief Konstante, die zur Auswertung des gewhlten Positronenspektrums bentigt wird, wird aus der Datendatei des Spektrums gelesen.
 */
double delta;
/*! \var double t0
    \brief Konstante, die zur Auswertung des gewhlten Positronenspektrums bentigt wird, wird aus der Datendatei des Spektrums gelesen.
 */
double t0;
/*! \var double sigma
    \brief Konstante, die zur Auswertung des gewhlten Positronenspektrums bentigt wird, wird aus der Datendatei des Spektrums gelesen.
 */
double sigma;
/*! \var int jmax
    \brief Die Anzahl der Modellterme der Modellfunktion
 */ 
int jmax;
/*! \var int tmax
    \brief Die Anzahl der maximalen Iterationen
 */
int tmax;
/*! \var int ndata
    \brief Anzahl der Messdaten
 */
int ndata;
/*! int overflow
    \brief Wird 1 gesetzt, falls ein Exponenten-Overflow aufgetreten ist.
 */
int overflow;
#endif
