/*! \file positronimp.h
    \brief Die Definition der Klasse \ref PositronImp

    Diese Datei wurde EINMAL automatisch durch Aufruf von

    uic -o positronimp.h -subdecl PositronImp positron.h positron.ui

    erzeugt.
    
    Anschlieend wurden die Ereignissbehandlungsroutinen mit dem bentigten Code
    gefllt.
*/
#ifndef POSITRONIMP_H
#define POSITRONIMP_H

#include "positron.h"
#include "aiplot.h"
#include <qprogressdialog.h>
#include <qthread.h>
#include <qfile.h>
#include <qmessagebox.h>
#include <qtextstream.h>
#include <qlistbox.h>
#include <qtabwidget.h>
#include <qcombobox.h>
#include <qtextbrowser.h>
#include <qtable.h>
#include <qspinbox.h>
#include <qdir.h>
#include <qslider.h>

/*! \def SPEKTRUMMAX
    \brief definiert die Anzahl der maximal zu ladenden Spektren
*/
#define SPEKTRUMMAX 50

/*! \def VOREINSTELLUNGENMAX
    \brief definiert die maximale Anzahl von vordefinierten
    Startwertkombinationen, die pro Spektrum geladen werden knnen.
*/
#define VOREINSTELLUNGENMAX 20


/*! \class PositronImp
    \brief Die Klasse des Hauptfensters
    
    Die Klasse des Hauptfensters ist abgeleitet von der von uic (User Interface Compiler)
    automatisch erzeugten Klasse Positron welche in der Datei positron.h definiert wird.
*/
class PositronImp : public Positron
{ 
    Q_OBJECT

public:

    /*! \fn PositronImp( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
        \brief Der Standardkonstruktor des Hauptfensters

         \param parent Die Parent-Klasse vom Typ QWidget
         \param name Der Name
         \param fl Hier knnten noch einige flags angegeben werden
    */
    PositronImp( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );

    /*! \fn ~PositronImp();
        \brief Der Destruktor des Hauptfensters

        Hier wird Speicher wieder freigegeben
    */
    ~PositronImp();

public slots:

     /*! \fn void Beenden_slot();
     \brief Die Ereignisbehandlungsroutine des Beenden-Buttons

     Dieser Slot wird aufgerufen, wenn der Benutzer auf den Beenden-Button drckt.

     Dadurch wird die Applikation beendet.
    */
    void Beenden_slot();

    /*! \fn void Hilfe_slot();
     \brief Die Ereignisbehandlungsroutine des Hilfe-Buttons

     Dieser Slot wird aufgerufen, wenn der Benutzer auf den Hilfe-Button drckt.

     Dadurch wird der Help-Browser der Applikation geffnet.
    */
    void Hilfe_slot();

    /*! \fn void Einfuehrung_Weiter_slot();
        \brief Die Ereignisbehandlungsroutine des Weiter-Buttons auf dem Einfhrungs-Tab

        Dieser Slot wird aufgerufen, wenn der Benutzer auf den Weiter-Button auf
        der Einfhrungsseite drckt.

        Der Benutzer kommt dadurch auf die nchste Seite, auf der er ein Spektrum auswhlen
        kann.
    */
    void Einfuehrung_Weiter_slot();

    /*! \fn Auswahl_EigeneSpektren_slot(int nr);
        \brief Die Ereignisbehandlungsroutine, die aufgerufen wird, wenn der Benutzer ein von
        ihm selbst simuliertes Spektrum aus der ListBox auswhlt.
        \param nr Die Nummer des ausgewhlten Spektrums.

        Wenn diese Routine aufgerufen wird, wird das aktuell ausgewhlte Spektrum aus der Datei geladen
        und die Grafik upgedatet.
    */
    void Auswahl_EigeneSpektren_slot(int nr);

    /*! \fn Auswahl_VordefinierteSpektren_slot(int nr);
        \brief Die Ereignisbehandlungsroutine, die aufgerufen wird, wenn der Benutzer ein 
         vordefiniertes Spektrum aus der ListBox auswhlt.
        \param nr Die Nummer des ausgewhlten Spektrums

        Wenn diese Routine aufgerufen wird, wird das aktuell ausgewhlte Spektrum aus der Datei geladen
        und die Grafik upgedatet.
    */
    void Auswahl_VordefinierteSpektren_slot(int nr);
 
    /*! \fn void Auswahl_Weiter_slot();
        \brief Die Ereignisbehandlungsroutine des Weiter-Buttons auf dem Auswahl-Tab

        Dieser Slot wird aufgerufen, wenn der Benutzer auf den Weiter-Button auf
        der Auswahlseite drckt.

        Der Benutzer kommt dadurch auf die nchste Seite, auf der er die Startwerte der 
        Gauss-Newton-Marquardt-Iteration auswhlen kann.
    */
    void Auswahl_Weiter_slot();

    /*! \fn void Auwahl_SpektrumSimulieren_slot();
        \brief Die Ereignisbehandlungsroutine des Neus Spektrum simulieren-Buttons auf dem Auswahl-Tab

        Dieser Slot wird aufgerufen, wenn der Benutzer auf den Neues Spektrum Simulieren-Button
        auf der Auswahlseite drckt.

        Dadurch wird ein Dialogfenster geffnet, in der der Benutzer die Werte fr das von ihm
        simulierte Spektrum vorgeben kann.
    */
    void Auwahl_SpektrumSimulieren_slot();
    
   /*! \fn void Startwerte_VordefinierteStartwerte_slot(int nr);
        \brief Die Ereignisbehandlungsroutine der Combobox auf der Startwerteseite
        aus der die vordefinierten Startwertkombinationen ausgewhlt werden knnen.

        \param nr Die Nummer der ausgewhlten Startwertkombination.

        Dieser Slot wird aufgerufen, wenn der Benutzer eine neue Startwertkombination auswhlt.

        Dadurch wird die Anzahl der Modellterme angepasst, und die Startwerte werden in die Tabelle
        eingetragen. 
    */
    void Startwerte_VordefinierteStartwerte_slot(int nr);

   /*! \fn void Startwerte_AnzahlderIterationen_slot(int nr);
        \brief Die Ereignisbehandlungsroutine der SpinBox auf der Startwerteseite, 
        in der der Benutzer die maximale
        Anzahl der Iterationen des Gauss-Newton-Marquardt-Prozesses festlegen kann.

        \param nr Die ausgewhlte Anzahl der maximalen Iterationen

        Dieser Slot wird aufgerufen, wenn der Benutzer die Anzahl der maximalen Iterationen
        verndert.
    */
    void Startwerte_AnzahlderIterationen_slot(int nr);

    /*! \fn void Startwerte_AnzahlderModellterme_slot(int nr);
        \brief Die Ereignissbehandlunsroutine der Combobox, aus der der Benutzer die Anzahl
        der Modellterme auswhlen kann.

        \param nr Die Nummer des Eintrags, den er aus der Combobox ausgewhlt hat.

        Dieser Slot wird aufgerufen, wenn der Benutzer die Anzahl der Modellterme verndert.
       
        Dadurch wird die Startwerttabelle gelscht, und der Benutzer kann neue Startwerte
        hndisch eintragen.
    */
    void Startwerte_AnzahlderModellterme_slot(int nr);

    /*! \fn void Startwerte_AuswertungStarten_slot();
        \brief Die Ereignissbehandlungsroutine des Auswertung-Starten-Buttons auf der
        Startwertseite

        Dieser Slot wird aufgerufen, wenn der Benutzer auf den Auswertung-Starten-Button drckt.
        
        Dadurch wird der Gauss-Newton-Marquardt-Prozess gestartet.
    */
    void Startwerte_AuswertungStarten_slot();

    /*! \fn void Startwerte_Genauigkeit_slot(int nr);
        \brief Die Ereignisbehandlungsroutine der Spinbox in der der Benutzer die gewnschte
        Genauigkeit der Auswertung auswhlen kann

        \param nr Die Genauigkeit, die der Benutzer ausgewhlt hat,
    */
    void Startwerte_Genauigkeit_slot(int nr);
    
    /*! \fn void Animation_Slider_slot(int nr);
        \brief Die Ereignisbehandlungsroutine des Sliders auf der Animationsseite

        \param nr Der Frame der Animation, der angezeigt werden soll.
    */
    void Animation_Slider_slot(int nr);

    /*! \fn void Animation_Play_slot();
        \brief Die Ereignisbehandlungsroutine des Play-Buttons auf der Animationsseite

        Dieser Slot wird aufgerufen, wenn der Benutzer den Play-Button auf der Animationsseite
        drckt.
        
        Dadurch wird die Animation abgespielt.
    */  
    void Animation_Play_slot();
    /*! \fn void Animation_Stop_slot()
         \brief Die Ereignisbehandlungsroutine des Stop-Buttons auf der Animationsseite
         
         Dieser Slot wird aufgerufen, wenn der Benutzer den Stop-Button auf der Animationsseite
         drckt.
  
         Dadurch wird die Animation beendet.
    */
    void Animation_Stop_slot();
    /* \fn void Ergebnisse_Kovarianz_slot();
       \brief Wird ausgelst, wenn der Benutzer auf den Kovarianzmatrix-Button auf der Ergebnissseite drckt, ffnet den Kovarianzmatrixdialog
     */
    void Ergebnisse_Kovarianz_slot();
    /*! \fn void Ergebnisse_Weiter_slot();
        \brief Wird ausgelst, wenn der Benutzer auf den Weiter-Button auf der Ergebnissseite drckt.
     */
    void Ergebnisse_Weiter_slot();
private:
    /*! \var int AnzahldervordefiniertenSpektren;
        \brief Gibt die Anzahl der vordefinierten Spektren an. 

        Wird beim Starten des Programms bestimmt, in dem festgestellt wird, wieviele
        Dateien mit der Endung .dat im daten-Verzeichnis liegen.
    */
    int AnzahldervordefiniertenSpektren;

    /*! \var int AnzahlderBenutzerSpektren;
        \brief Gibt die Anzahl der Benutzerspektren an.

        Wird beim Starten des Programms bestimmt, in dem festgestellt wird, wieviele
        Dateien mit der Endung .usr im daten-Verzeichnis liegen.
        
        Falls der Benutzer ein neues Spektrum simuliert, wird die Variable demgem erhht.
    */
    int AnzahlderBenutzerSpektren;

    /*! \var int AnzahlderVoreinstellungen;
        \brief Gibt die Anzahl der vordefinierten Startparameterkombinationen des ausgewhlten Spektrums
        an.

        Wird bestimmt, indem diese aus der Datei geladen wird. 
    */
    int AnzahlderVoreinstellungen[2*SPEKTRUMMAX];
    /*! \var int Voreinstellunggewaehlt
        \brief Gibt an ob vordefinierte Startwerte gewhlt wurden.
     */
    int Voreinstellunggewaehlt;
    /*! \var int Spektrumgewaehlt;
        \brief Gibt die Nummer des gerade gewhlten Spektrums an
     */
    int Spektrumgewaehlt;
    /*! \var int VordefinierteAnzahlderZerfallsterme
        \brief Enthlt zu jedem Spektrum die vordefinierte Anzahl der Zerfallsterme
     */
    int VordefinierteAnzahlderZerfallsterme[2*SPEKTRUMMAX];
    /*! \var QString voreinstellungennamen[VOREINSTELLUNGENMAX];
        \brief Feld, dass die Namen der vordefinierten Startwertkombinationen enthlt
    */
    QString voreinstellungennamen[VOREINSTELLUNGENMAX];
    /*! \var  QString voreinstellungenergebnisstext[VOREINSTELLUNGENMAX];
        \brief Feld, dass den zu den Voreinstellungen passenden Text, welcher auf der Ergebnissseite angezeigt wird, enthlt.
    */
    QString voreinstellungenergebnisstext[VOREINSTELLUNGENMAX];
    /*! \var int voreinstellungenjmax[VOREINSTELLUNGENMAX];
        \brief Feld, dass die zu den vordefinierten Startwertkombinationen gehrige Anzahl der Zerfallsterme enthlt.
     */
    int voreinstellungenjmax[VOREINSTELLUNGENMAX];
    /*! \var  double voreinstellungenstartwerte[VOREINSTELLUNGENMAX][12];
        \brief Feld, dass die vordefinierten Startwertkombinationen enthlt.
     */
    double voreinstellungenstartwerte[VOREINSTELLUNGENMAX][12];

    /*! \var QString dateinamen[2*SPEKTRUMMAX];
        \brief Enthlt die Dateinamen der Spektren im Verzeichnis Daten
     */
    QString dateinamen[2*SPEKTRUMMAX];
    /*! \var  QString spektrumnamen[2*SPEKTRUMMAX];
        \brief Enthlt die Namen der geladenen Spektren
     */
    QString spektrumnamen[2*SPEKTRUMMAX];
    /*! \fn void init();
        \brief Ruft alle anderen Initialisierungsroutinen auf.
     */
    void init();
    /*! \fn void initEinfuehrung();
        \brief Initialisiert die Einfhrungsseite des Tabcontrols
    */
    void initEinfuehrung();
    /*! \fn void aktualisiereAuswahlGraphik(int nr);
        \brief Nimmt eine Aktualisierung der Graphik auf dem AuswahlTab vor
        \param nr Gibt die Nummer des ausgewhlten Spektrums an.
     */
    /*! \fn int initFonts();
        \brief Nimmt eine Neuinitialisierung aller Fonts der Applikation vor.
     */
    int initFonts();
    void aktualisiereAuswahlGraphik(int nr);

    /*! \var double *xmess
       \brief Der Vektor der x-Koordinaten der Messpunkte
    
       Da in diesem Fall die x-Koordinaten die Kanalnummern darstellen, ein Feld
       von der Gre 1-\ref ndata mit dem Inhalt 1-\ref ndata
    */
    /*! \var double *ymess
      \brief Der Vektor der y-Koordinaten der Messpunkte
  
      Ein Feld von der Gre 1-\ref ndata. Der Inhalt wird aus der Datendatei des Spektrums geladen.
    */

    /*! \var double *sig
      \brief Der Vektor der Standardabweichung der y-Koordinaten der Messpunkte
  
      Ein Feld von der Gre 1-\ref ndata. Da hier eine Poisson - Verteilung vorliegt, wird
      dieser einfach aus der Wurzel der \ref ymess - Elemente bestimmt.
    */
    /*! \var double *yplot
         \brief Der Vektor der auf dem AuswahlTab geplottet wird.
      
         Ein Feld von der Gre 1-\ref ndata. Wird aus dem natrlichen Logarithmus der \ref ymess -Elemente bestimmt.
    */
    /*! \var double alamda
         \brief aktueller Marquardt'scher Parameter
    */
    double *xmess,*ymess,*sig,*yplot,alamda;
    /*! \var double *astart
         \brief Der Vektor mit den Startwerten der Fitparameter.

         Da diese bei jeder Iteration gendert werden, werden die Anfanfsparameter hier zwischengespeichert,
         damit sie in der Animation verwendet werden knnen.
    */
    double *astart;
    /*! \var double residuummax
         \brief Das Maximum der Residuenwerte, wird zum festlegen der Grenzen fr den Residuumplot verwendet.
    */
    /*! \var double residuummin
          \brief Das Maximum der Residuenwerte, wird zum festlegen der Grenzen fr den Residuumplot verwendet.
   */
    double residuummax,residuummin;
    /*! \var int tatit
         \brief Die Anzahl der tatschlich bentigten Iterationen
   */
    int tatit;
    /*! \fn void initAuswahl()
         \brief In dieser Funktion wird der AuswahlTab initialisiert.
    */
    void initAuswahl();
    /*! \fn void initStartwerte()
         \brief In dieser Funktion wird der StartwerteTab initialisiert.
    */
    void initStartwerte();
    /*! \fn void aktualisiereAnimation(int frame)
         \brief aktualisiert die beiden Graphen auf dem AnimationsTab, wird beim Abspielen der Animation aufgerufen
    */
    void aktualisiereAnimation(int frame);
    /*! \fn  int leseDateiNamen()
         \brief Liest die Dateinamen aus dem daten-Verzeichnis
    */
    int leseDateiNamen();
    /*! \fn int leseSpektrumName(int nummer)
         \brief Liest den Spektrumnamen aus der gerade ausgewhlten Datei
    */
    int leseSpektrumName(int nummer);
    /*! \fn int leseSpektrum(int nummer)
         \brief Liest das gerade ausgewhlte Spektrum aus der Datei
   */
    int leseSpektrum(int nummer);
    /*! \var AIPlot *AnimationGraphic
         \brief Die Animation der Annherung der Fitkurve zu den Messpunkten
    */
    AIPlot *AnimationGraphic;
    /*! \var AIPlot *AnimationGraphic2
         \brief Die Animation des Residuenverlaufs
    */
    AIPlot *AnimationGraphic2;
    /*! \var AIPlot *AuswahlGraphic
         \brief Die Grafik auf dem AuswahlTab, in der der Logarithmus der Messpunkte angezeigt wird.
    */
    AIPlot *AuswahlGraphic;
    /*! \var AIPlot *ChisqGraphic
         \brief Die Grafik, in der der Verlauf von chisq angezigt wird.
    */
    AIPlot *ChisqGraphic;
 protected:
    bool event( QEvent* );
};

#endif // POSITRONIMP_H

